;AirConditionerTimer.asm

;Modified old AirConditionerTimer.asm code to follow scheme used for Kitchen Timer 4/29/06
;That is hold down counts add one hour, dahs add 10 minutes, and dits add one minute.

;Current draw at idle = approx 1.5 ma (@5.22 volts)
;Short Dit Touch = one minute addition
;Longer Dah Touch = 10 minute additions
;Hold down count - one hour addition
;Internal oscillator in ATTINY2313 is running micro-controller at approx. 1MHz

;June 7, 2006

.include "tn2313def.inc"

rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt		;one second interval interrupt
rjmp	START1
rjmp	START1

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ser		r16			;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output

clr		r16
out		PortB,r16	;LED is off when port pin is low

ldi		R16,254		;Set bit 0, as input ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 0 as input
out		PORTD,R16	;execute to set out ports high, and in port with
					; no input resistor

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
					;and bits 0,1,2 select prescaling source as 64 of clock

ldi		r16,61
out		OCR1AH,r16
ldi		r16,112		;default selected for one second interval interrupt (70 deg)
out		OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
					;set for 1,000,000 clock frequency

clr		r18			;clear all flags

bset	7			;enable global interrupts

rcall	Delay5sec	;allow touch plate circuit to settle (five second delay)
cbi		PortB,7		;off relay B7(128)


;-----------------------------

WaitLoop:		;Power down and loop to await interrupt

sbis	PIND,0		;skip if switch is up
rcall	Decode

rjmp	WaitLoop


;----------------------------------

Decode:		;Interrupt Routine: Decodes touches from touch plate

bclr	7			;off global interrupts
cbr		r18,1		;clear minute flag, r18,0(1)
clr		r24			;zero seconds counter
clr		r25			;zero 10 second counter
clr		r26
clr		r27			;zero Time Count register pair
bset	7			;turn global interrupts back on

rcall	ButtonPush3

cpi		r17,95		;  .- .- - - - -    toggle one minute flash on and off
breq	D1

cpi		r17,85		;  .- .- .- .-       enter new low byte
breq	D2

cpi		r17,87		;  .- .- .- - -      flash out low byte
breq	D3

rjmp	Countdown	;it is a timing function - start timing


D1:

rcall	ToggleLED
ret


D2:

rcall	NewDivideNumber
ret


D3:

in		r23,OCR1AL			;fetch current low byte divider value
rcall		LEDFlashByte
ret


DecodeEnd:

cbi		PortB,7			;clear bit 7(128) - relay bit - off relay
ret						;return


;-----------------------------------------

Countdown:

clr		r24			;zero seconds counter
clr		r25			;zero 10 second counter
sbi		PortB,7		;on relay, B7(128)

Countdown1:

sbrc	r18,3	;skip if r18,3(8), countdown 2 flag is clear
rjmp	CD2


CD1a:

sbic	PIND,0	;skip if switch is down D0(1)
rjmp	CD1


CD1b:

rcall	Delay40th
sbis	PIND,0 			;skip if switch is up
rjmp	CD1b

sbr		r18,8 			;set countdown2 flag, r18,3(8)

ldi		r16,1
rcall	VariableLED		;flash LED once
clr		r23				;clear r23 delay counter


CD1:

sbrs	r18,2			;skip if r18,2(4), countdownUP flag is set
rjmp	Countdown1

cbr		r18,4			;clear r18,2(4), countdownUP flag
rjmp	DecodeEnd


CD2:

inc		r23
rcall	Delay40th

sbis	PIND,0			;skip if switch is up
rjmp	CD2a

cpi		r23,160
brlo	CD1

cbr		r18,8			;clear r18,3(8), countdown2 flag
rjmp	CD1


CD2a:

rcall	Delay40th
sbis	PIND,0			;skip if switch is up
rjmp	CD2a

cbr		r18,8			;clear r18,3(8), countdown2 flag
ldi		r16,2
rcall	VariableLED		;give two flashes of LED

bclr	7				;turn off interrupts
clr		r26
clr		r27				;clear countdown counter pair r26,r27
cbr		r18,4			;clear Countdown up flag, r18,2(4)
bset	7				;turn interrupts back on

rjmp	DecodeEnd


;-------------------------------

ClockCntDwnInt: 	;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$60,r16

sbr		r18,2		;set one second flag, bit 1(2) in r18 flags register

inc		r24			;increment seconds counter
cpi		r24,60		
brsh	CCD1
rjmp	CCDExit


CCD1:

clr		r24			;zero 60 second counter
sbr		r18,1		;set bit 0(1) in R18, set one minute flag


sbrs	r18,4		;skip if One Minute LED flag, r18,4(16) is set
rjmp	CCD1a

sbi		PortB,0		;on LED
rcall	Delay10		;1/100 second delay
cbi		PortB,0		;off LED


CCD1a:

mov		r16,r26
or		r16,r27
breq	CCDExit

sbiw	r26,1
brne	CCDExit

sbr		r18,4		;set CountdownUP flag - r18,2(4)


CCDExit:			;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r16

reti


;------------------------

ButtonPush3:		;"x" r26,27 register pair = TimeCount

clr		r17			;clear code entry register
clr		r20			;Holddown count
clr		r23			;zero universal delay counter
clr		r26			;TIME counter (r26, r27 register pair)
clr		r27			;clear timecount registers
clr		r28			;clear dit-dah count (r28, r29 reg. pair)
clr		r29


B1a:

clr		r25			;clear 4 second WAIT counter
clr		r24			;zero 60 second counter


B1:

inc		r23			;increment universal delay counter
rcall	Delay40th
cpi		r23,120
brsh	B3

sbis	PIND,0		;skip is switch is up
rjmp	B1a

cpi		r23,20
brsh	B1b			;it is dah count

adiw	r26,1		;add ONE to timecount register pair
adiw	r28,1		;add ONE to dit-dah counter pair

sbi		PortB,0		;On LED briefly
rcall	Delay10		
cbi		PortB,0

lsl		r17			;shift ZERO into code register r17, code register


B1c:

clr		r23


B2d:

sbic	PIND,0		;skip if switch down
rjmp	B5

clr		r23
rjmp	B1a


B1b:

adiw	r26,10		;add 10 to r26,27 timecount register (dah)
adiw	r28,10		;add 10 to dit-dah counter

sbi		PortB,0		;short on of LED
rcall	Delay20
cbi		PortB,0		;off LED

sec					;set carry flag
rol		R17			;rotate ONE into code register, r17

rjmp	B1c


B3:

sbi		PortB,0		;on PortB,0(1), LED
rcall	Delay25		;1/4 second delay
cbi		PortB,0		;off LED

adiw	r26,60		;add 60 minutes to timecount reg r26,27 (1 hour)
inc		r20			;add ONE to Holddown Counter

clr		r23			;clear universal delay counter


B4:

inc		r23
rcall	Delay40th

sbic	PIND,0		;skip if switch down
rjmp	B5

clr		r24			;zero 60 second counter
clr		r25			;zero 4 second wait counter
cpi		r23,30	
brsh	B3

rjmp	B4


B5:

sbrs	r18,1		;skip if r18,1(2) is set (second flag)
rjmp	B2d

cbr		r18,2		;clear r18,1(2) second flag
inc		r25			;increment r25, 10second counter

cpi		r25,4		;limit wait period to 4 seconds
brlo	B2d			;continue

ret


;----------------------------------------------

Delay40th:		; 1/40th second delay

push	r0
push	r31

clr		r0
ldi		r31,33		;put 16 into r31 (high byte) for countdown


D40a:

dec		r0
brne	D40a

dec		r31			;freq countdown = 1/40th second
brne	D40a

pop		r31
pop		r0

ret


;----------------------------------

Delay10:			;1/100 second delay

push	r0
push	r16
ldi		r16,6


D10a:

dec		r0
brne	D10a

dec		r16
brne	D10a

pop		r16
pop		r0

ret


;----------------------------------

Delay20:				;1/200 second delay

push	r0
push	r16
ldi		r16,13

rjmp	D10a


;-------------------------------------

Delay25:			;1/4 second delay

push 	r16
ldi		r16,10

rjmp	D75		;to end routine


;----------------------------------

Delay50:			;1/2 second delay

push	r16
ldi		r16,20		;1/2 second delay number

rjmp	D75


;---------------------------------------

Delay75:			;3/4 second delay

push	r16
ldi		r16,30


D75:

rcall	Delay40th
dec		r16
brne	D75

pop		r16
ret


;-------------------------------------

Delay100:			;ONE second Delay

push	r16
ldi		r16,40

rjmp	D75


;-------------------------------------

Delay5sec:			;5 second delay

push	r16
ldi		r16,200

D5SEC1:

rcall	Delay40th
dec		r16
brne	D5SEC1

pop		r16
ret

;------------------------------------

VariableLED:		;Flashes LED number of times in r16

sbi		PortB,0		;on LED
rcall	Delay25
cbi		PortB,0		;off LED
rcall	Delay25

dec		r16
brne	VariableLED

ret


;-----------------------------------

ToggleLED:			;toggles LED flag (for on and off each iteration)

sbrs	r18,4		;skip if LED flag, r18,4(16) is set
rjmp	TB1

cbr		r18,16		;clear flash flag, r18,4(16)
ldi		r16,2
rcall	VariableLED		;two LED flashes (on)
ret


TB1:	

sbr		r18,16		;set flash flag, r18,4(16)
ldi		r16,1
rcall	VariableLED		;flash once (off)

ret


;----------------------------------------------

NewDivideNumber:	;put in new divide number

ldi		r16,5	
rcall	VariableLED		;flash LED five times


ND1:

rcall	Delay40th
sbic	PIND,0		;skip if switch is down
rjmp	ND1

rcall	ButtonPush3

cpi		r20,1		;Error, go back without a write
brlo	ND2

ldi		r16,1
rcall	VariableLED		;flash LED once -- no entry made

ret


ND2:

mov		r22,r17		;put new divide byte
out		OCR1AL,r22	;put new number in low byte compare register

ldi		r16,2
rcall	VariableLED		;flash LED twice to signal a valid entry

ret


;--------------------------------------	

LEDFlashByte:			;r23 contains byte to be flashed out

clr		r16


BB1:

rol		r23
brcs	BB2

sbi		PortB,0		;on LED
rcall	Delay40th
cbi		PortB,0		;off LED


BB3:

inc		r16
rcall	Delay100		;One second delay

cpi		r16,8
brne	BB1

ret


BB2:

sbi		PortB,0		;on sonalert for long beep
rcall	Delay50		;1/2 second on time
cbi		PortB,0		;off sonalert
rjmp	BB3


;------------------------------------

